import QtQuick 2.15
import QtQuick.Controls 2.3
import QtQuick.Layouts 1.3
import Dcchat 1.0
import DCStyle 1.0

import "MessageArea.js" as MS

PanelView{
    // default property alias data: view.data
    id: mess_area
    required property int chat_id
    signal loadMoreData();
    property alias model: model
    color: MessStyle.mess.background
    Layout.fillWidth:  true
    ChatRoomModel {
        id:model
    }
    MessageModel {
        id:mess
        chatId: chat_id
    }
    Menu {
        id: menu
        MenuItem {
            text: "Clear Chat"
            onTriggered: {
                model.clearAllMsg()
            }
        }
        MenuItem {
            text: "Delete chat"
            onTriggered: MS.deleteChat()
        }
        MenuItem {
            text: "Leave Group"
            enabled:  model.isGroup()
            onTriggered:  {
                MS.leaveGroup()
            }
        }
        MenuItem {
            text:"Create Invite Qrcode"
            onTriggered: {
                ChatMediator.createQrCode(chat_id)
                MS.showView("popup/QrCodePopup.qml")
            }
        }
        MenuItem {
            text: "Send Qrcode"
            onTriggered:  {
                console.log("qr code is"+ChatMediator.getQrCode())
                local.text = ChatMediator.getQrCode()
            }
        }
    }
    ColumnLayout {
        anchors.fill: parent
        spacing: 5
        Rectangle {
            id: _header
            Layout.fillWidth: true
            height: 50
            color: MessStyle.user.banner
            z:2
            RowLayout {
                anchors.fill: parent
                spacing: 3
                ContactIcon {
                    $text : model.chatName()
                    Layout.fillWidth:  true
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            MS.showView("popup/GroupContact.qml",{ chat_id : chat_id })
                        }
                    }
                }
                /*
                TextIcon {
                    id: icon
                    size: _header.height
                    dptext: model.chatName()
                }
                Text {
                    id: banner
                    height: _header.height
                    color: "white"
                    text: model.chatName()
                    Layout.fillWidth:  true
                    verticalAlignment: Text.AlignVCenter
                    // horizontalAlignment: Text.AlignHCenter
                    MouseArea {
                        anchors.fill: parent
                        onClicked: {
                            MS.showView("popup/GroupContact.qml")
                        }
                    }

                }
                */
                Button {
                    text:  "Menu"
                    id: _btn_menu
                    height: _header.height
                    onClicked: {
                        // show on last
                        menu.popup(_header.width - menu.width,_header.height)
                    }
                }
            }

        }

        ListView {
            id:view
            Layout.fillHeight: true
            Layout.fillWidth: true
            model : model
            spacing: 3
            delegate: MessageItem {
                width: mess_area.width
            }
            cacheBuffer:20
            //for ScrollBar
            ScrollBar.vertical: ScrollBar {
                id: scroll_bar
                parent: parent
            }

            TextIcon {
                id: f_btn
                x: parent.width - width - 40
                y: parent.height - height - 40
                size: 40
                visible: ! view.atYEnd
                // ma unicode cua mui ten
                dptext:  String.fromCodePoint(0x2193)
                MouseArea {
                    anchors.fill: parent
                    onClicked:  {
                        view.positionViewAtEnd()
                    }
                }
            }
        }


        Rectangle {
            // send bare
            z:2
            Layout.fillWidth: true
            height: 30
            visible: model.active
            RowLayout {
                anchors.fill: parent
                TextArea {
                    id : local
                    height: parent.height
                    Layout.fillWidth: true
                }
                Button {
                    id : btn
                    text: "send"
                    onClicked: {
                        mess.sendText(local.text)
                        local.text = ""
                        if(! view.atYEnd)
                            view.positionViewAtEnd()
                    }
                }
            }
        }
    }
    // load more message when scroll at end
    Connections {
        target: view
        function onMovementEnded(){
            if(view.atYBeginning )
            {
                MS.loadMoreData()
            }

        }
    }
    // handle when have incomming mess
    function addmodel() {
        view.model = model
    }
    Connections {
        target: model
        function onUpdateData(){
            if( view.atYEnd )
            {
                Qt.callLater(MS.viewChange)
            }

        }
    }

    Component.onCompleted:
    {
        color= MessStyle.mess.background
        view.positionViewAtEnd()
    }
}
