#include "chatui.h"
#include "task.h"
#include <deltachat/deltacpp.h>
ChatUi::ChatUi(Task* parent)
    : QObject { parent }
    , ts(stdout)
{
    connect(parent, &Task::processInput, this, &ChatUi::proccsesInput);
}

void ChatUi::proccsesInput(const QStringList& qm)
{
    if (qm.at(0) == "help") {
        showHelp();
        return;
    }
    if (qm.at(0) != "chat") {
        return;
    }
    if (qm.size() < 2)
        return;
    auto cmd = qm.at(1);
    if (cmd == "show") {
        showChat();
    } else if (cmd == "create") {
        createWithContact(qm);
    } else if (cmd == "remove") {
        if (qm.size() == 3)
            chatRemove(qm.at(2));
    } else if (cmd == "group") {
        if (qm.size() < 4)
            return;
        auto cmd2 = qm.at(3);
        if (cmd2 == "create") {
            createGroup(qm);
        } else if (cmd2 == "add") {
            addUser(qm);
        } else if (cmd2 == "rm") {
            rmUser(qm);
        }
    } else if (cmd == "send") {
        chatSend(qm);
    } else if (cmd == "list" && qm.size() == 3) {
        listMess(qm.at(2));
    } else {
        ts << "Wrong command , using help for detail";
    }
}

void ChatUi::showHelp()
{
    ts << "Chat Command\n";
    ts << "\tchat show : show avaible chat and id\n";
    ts << "\tchat create <contact_id> : create a chat with contact id\n";
    ts << "\tchat send <chat_id> \"mess\": send message to chat with id\n";
    ts << "\tchat list <chat_id>: see last 10 message in chat with id\n";
    ts << "\tchat remove <chat_id> : remove chat id\n";
    ts << "\tchat group create <name>: create a group chat with name\n";
    ts << "\tchat group add <chat_id> <contact_id>: add contact id to group chat\n";
    ts << "\tchat group rm <chat_id> <contact_id>: remove contact id from group chat";
    ts << Qt::endl;
}

void ChatUi::showChat()
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    auto chatlist = ctx->getChatlist(0, "");
    int d = chatlist->getChatCount();
    for (int i = 0; i < d; i++) {
        int id = chatlist->getChatId(i);
        auto chat = ctx->getChat(id);
        ts << "\tChat Id: " << id << "\twith name " << chat->getName() << Qt::endl;
    }
}

void ChatUi::createWithContact(const QStringList& qm)
{
    if (qm.size() < 3) {
        ts << "\tNot enought arument" << Qt::endl;
        return;
    }
    int id = qm.at(2).toInt();
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    int rid = ctx->createChatByContactId(id);
    ts << "\tNew chat is created with Id: " << id << Qt::endl;
}

void ChatUi::createGroup(const QStringList& qm)
{
    auto name = qm.at(3);
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    int id = ctx->createGroupChat(name, false);
    ts << "\tNew group chat is created with Id: " << id << Qt::endl;
}

void ChatUi::chatRemove(const QString& id)
{
    int cid = id.toInt();
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    ctx->deleteChat(cid);
    ts << "\tChat id " << id << "is delete" << Qt::endl;
}

void ChatUi::addUser(const QStringList& qm)
{
    if (qm.size() != 5)
        return;
    int chat_id = qm.at(3).toInt();
    int contact_id = qm.at(4).toInt();
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    ctx->addContactToChat(chat_id, contact_id);
    ts << "\tContact " << contact_id << "is added to group" << Qt::endl;
}

void ChatUi::rmUser(const QStringList& qm)
{
    if (qm.size() != 5)
        return;
    int chat_id = qm.at(3).toInt();
    int contact_id = qm.at(4).toInt();
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    ctx->removeContactFromChat(chat_id, contact_id);
    ts << "\tContact " << contact_id << "is remove from group" << Qt::endl;
}

void ChatUi::chatSend(const QStringList& qm)
{
    if (qm.size() != 4) {
        ts << "wrong command" << Qt::endl;
        return;
    }
    int chat_id = qm.at(2).toInt();
    auto mess = qm.at(3);
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    ctx->sendTextMessage(chat_id, mess);
}

void ChatUi::listMess(const QString& id)
{
    int chat_id = id.toInt();
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    auto list = ctx->getMsgIdList(chat_id);
    int v = list.count() - 10;

    if (list.count() == 0) {
        ts << "\tNo Mess in Chat " << Qt::endl;
        return;
    }
    if (v < 0)
        v = 0;
    for (; v < list.count(); v++) {
        auto mid = list.at(v);
        auto msg = ctx->getMessage(mid);
        ts << "\tMess: " << msg->getText() << Qt::endl;
    }
}
