#ifndef ACCOUNT_H
#define ACCOUNT_H

#include "deltachat.h"
#include <QList>
#include <QObject>
#include <QSharedPointer>
#include <memory>

namespace dc {
class Context;
class Event;
class EventEmitter;

class Account : public QObject {
    Q_OBJECT
private:
    dc_accounts_t* m_accounts;

    static std::shared_ptr<Account> inst;

    EventEmitter* m_emit;

public:
    Account(QString datapath);
    ~Account();
    // init some proprey of deltachat
    static void init(QString datapath);
    // close library
    static void close();

    static std::shared_ptr<Account> getInstance();

    // warper funtion
    void startIo();
    void stopIo();

    // network
    void maybeNetWork();
    void maybeNetWorkLost();

    // fetch
    void backgroundFetch(uint64_t timeout);

    // account
    std::shared_ptr<Context> getAccount(uint32_t id);
    std::shared_ptr<Context> getSelectedAccount();
    void setSelectAccount(uint32_t id);
    void removeAccount(uint32_t id);
    uint32_t addAcount();

    QList<uint32_t> getAll();
    // custom funtion

    void startEventLoop();
    std::shared_ptr<Context> getUser(const QString& addr);
public slots:
    void processEvent(QSharedPointer<Event> evt);
signals:

    void connectivityChange(uint32_t accountId, int status);
    void removeContactFromChat(uint32_t accountId, int chatId, int contact_id);
    void chatDeleted(uint32_t accountId, int chatId);
    void chatModified(uint32_t accountId, int chatId);
    void chatListItemChange(uint32_t accountId, int chatId);
    void configureProgress(uint32_t accountId, int progress, QString comment);
    void imexProgress(uint32_t accountId, int progress);
    void incomingMessage(uint32_t accountId, int chatId, int msgId);
    void messagesChanged(uint32_t accountId, int chatId, int msgId);
    void messagesNoticed(uint32_t accountId, int chatId);
    void messagesDelevery(uint32_t accountId, int chatId, int msgId);
    void messageFalse(uint32_t accountId, int chatId, int msgId);
    // signal war sen by another application to notify we send new mess
    void outGoingMessage(uint32_t accountId, int chatId, int msgId);
};
} // namespace dc

#endif // ACCOUNT_H
