#ifndef CONTEXT_H
#define CONTEXT_H

#include <QObject>
#include <QVariantList>
#include <deltachat.h>
#include <memory>

namespace dc {

class Message;
class Contact;
class Chat;
class Chatlist;
class Lot;

class Context : public QObject {
    Q_OBJECT
private:
    dc_context_t* m_context;

    // for gui chat

public:
    Context(dc_context_t* mctx);
    ~Context();
    void configure();
    bool isConfigured() const;
    QString getInfo();
    void startIo();
    void stopIo();
    void maybeNetwork();
    int getConnectivity();
    /* chat untion */
    std::shared_ptr<Chatlist> getChatlist(int flags, QString query);
    uint32_t createChatByContactId(uint32_t contactId);
    void setChatVisibility(uint32_t chatId, int visibility);
    void deleteChat(uint32_t chatId);
    void blockChat(uint32_t chatId);
    void acceptChat(uint32_t chatId);
    QString getChatEncrinfo(uint32_t chatId);
    uint32_t getChatEphemeralTimer(uint32_t chatId);
    std::shared_ptr<Chat> getChat(uint32_t chatId);
    /* contatct function */
    QList<int> getContacts(uint32_t flags, QString query);
    QList<int> getMsgIdList(uint32_t chatId, int flag = 0);
    int getFreshMsgCnt(uint32_t chatId);
    void marknoticedChat(uint32_t chatId);
    void markseenMsgs(const QVector<uint32_t>& msg_ids);
    std::shared_ptr<Message> getMessage(uint32_t msgId);
    std::shared_ptr<Contact> getContact(uint32_t contactId);
    void deleteMsgs(const QVector<uint32_t>& msg);
    // contact
    uint32_t createContact(QString name, QString addr);
    int deleteContact(uint32_t contact_id);

    uint32_t sendTextMessage(uint32_t chatId, QString textToSend);
    uint32_t sendMessage(uint32_t chatId, std::shared_ptr<Message>& message);

    void setDraft(uint32_t chatId, std::shared_ptr<Message>& message);
    std::shared_ptr<Message> getDraft(uint32_t chatId);
    QString getBlobdir();
    bool setConfig(QString key, QString value);
    QString getConfig(QString key);
    bool setChatMuteDuration(uint32_t chatId, int64_t duration);
    QString getMessageInfo(uint32_t msgId);
    QString getMessageHtml(uint32_t msgId);
    std::shared_ptr<Message> newMessage(int viewtype);

    void importBackup(QString tarfile);
    // qr code funtion
    std::shared_ptr<Lot> checkQr(const QString& qr);
    // create a image to scan
    QString getSecurejoinQrSvg(uint32_t id = 0);
    // create qr
    QString getSecurejoinQr(uint32_t id = 0);
    uint32_t joinSecurejoin(QString qrcode);

    /* for chat grop*/
    uint32_t createGroupChat(const QString& name, bool secure = true);
    int removeContactFromChat(uint32_t chat_id, uint32_t contact_id);
    int addContactToChat(uint32_t chat_id, uint32_t contact_id);
    QList<uint32_t> getChatContacts(uint32_t chat_id);
    std::shared_ptr<Lot> chatlistGetSummary2(uint32_t chat_id, uint32_t msg_id);
    /* id */
    uint32_t getId();
};

} // namespace dc

#endif // CONTEXT_H
