#include "message.h"
#include <QFile>

namespace dc {
Message::Message(dc_msg_t* mgs)
{
    this->m_message = mgs;
}

Message::~Message()
{
    dc_msg_unref(m_message);
}

uint32_t Message::getId()
{
    return dc_msg_get_id(m_message);
}

uint32_t Message::getFromId()
{
    return dc_msg_get_from_id(m_message);
}

uint32_t Message::getChatId()
{
    return dc_msg_get_chat_id(m_message);
}

int Message::getViewtype()
{
    return dc_msg_get_viewtype(m_message);
}

int Message::getState()
{
    return dc_msg_get_state(m_message);
}

QDateTime Message::getTimestamp()
{
    return QDateTime::fromSecsSinceEpoch(dc_msg_get_timestamp(m_message), Qt::UTC);
}

bool Message::saveAttach(QString dest)
{
    if (!dest.isNull()) {
        char* file = dc_msg_get_file(m_message);
        QString fileUrl { file };
        dc_str_unref(file);

        // We need to replace, so we delete the file first.
        if (QFile::exists(dest)) {
            if (!QFile::remove(dest))
                return false;
        }
        return QFile::copy(fileUrl, dest);
    } else {
        return false;
    }
}

void Message::setText(QString text)
{
    QByteArray utf8Text = text.toUtf8();
    return dc_msg_set_text(m_message, utf8Text.constData());
}

QString Message::getText()
{
    char* text = dc_msg_get_text(m_message);
    QString result { text };
    dc_str_unref(text);
    return result.trimmed();
}

// void Message::setFile(QString file)
// {
//     QByteArray utf8File = file.toUtf8();
//     return dc_msg_set_file(m_message, utf8File.constData(), NULL);
// }

QString Message::getSubject()
{
    char* subject = dc_msg_get_subject(m_message);
    QString result { subject };
    dc_str_unref(subject);
    return result;
}

void Message::setSubject(QString sub)
{
    QByteArray data = sub.toUtf8();
    dc_msg_set_subject(m_message, data.constData());
}

QString Message::getFile()
{
    char* file = dc_msg_get_file(m_message);
    QString result { file };
    dc_str_unref(file);
    return result;
}

QString Message::getFilename()
{
    char* filename = dc_msg_get_filename(m_message);
    QString result { filename };
    dc_str_unref(filename);
    return result;
}

QString Message::getFilemime()
{
    char* filemime = dc_msg_get_filemime(m_message);
    QString result { filemime };
    dc_str_unref(filemime);
    return result;
}

int Message::getWidth()
{
    return dc_msg_get_width(m_message);
}

int Message::getHeight()
{
    return dc_msg_get_height(m_message);
}

bool Message::getShowPadlock()
{
    return dc_msg_get_showpadlock(m_message);
}

bool Message::getIsBot()
{
    return dc_msg_is_bot(m_message);
}

QString Message::getOverrideSenderName()
{
    char* name = dc_msg_get_override_sender_name(m_message);
    QString result { name };
    dc_str_unref(name);
    return result;
}

bool Message::isInfo()
{
    return dc_msg_is_info(m_message);
}

QString Message::getError()
{
    char* error = dc_msg_get_error(m_message);
    QString result { error };
    dc_str_unref(error);
    return result;
}

bool Message::hasHtml()
{
    return dc_msg_has_html(m_message);
}

QString Message::getQuotedText()
{
    char* text = dc_msg_get_quoted_text(m_message);
    QString result { text };
    dc_str_unref(text);
    return result;
}

std::shared_ptr<Message> Message::getQuotedMessage()
{
    dc_msg_t* quote = dc_msg_get_quoted_msg(m_message);
    if (quote) {
        auto ret = std::shared_ptr<Message>(new Message(quote));
        return ret;
    } else {
        return NULL;
    }
}

int Message::getInfoType()
{
    return dc_msg_get_info_type(m_message);
}

} // namespace dc
