#include "chatlistmodel.h"
#include "chatcore.h"
#include <QDebug>

ChatListModel::ChatListModel(QObject* parent)
    : QAbstractListModel { parent }
{
    auto acc = dc::Account::getInstance();
    auto ctx = acc->getSelectedAccount();
    if (ctx) {
        auto plist = ctx->getChatlist(0, "");
        for (int i = 0; i < plist->getChatCount(); i++) {
            list.append(plist->getChatId(i));
        }
    }
    connect(acc.get(), &dc::Account::incomingMessage, this, &ChatListModel::onIncomingMessage);
    connect(acc.get(), &dc::Account::chatDeleted, this, &ChatListModel::onDeleteChat);
    connect(acc.get(), &dc::Account::connectivityChange, this, &ChatListModel::onConnectyChange);
    connect(acc.get(), &dc::Account::messagesNoticed, this, &ChatListModel::onMsgNotice);
    connect(acc.get(), &dc::Account::chatListItemChange, this, &ChatListModel::onChatListItemChange);
}

int ChatListModel::rowCount(const QModelIndex& parent) const
{

    return list.size();
}

QHash<int, QByteArray> ChatListModel::roleNames() const
{
    QHash<int, QByteArray> ret;
    ret[ListRole::ChatID] = "$chatid";
    ret[ListRole::Username] = "$name";
    ret[ListRole::Count] = "$count";
    return ret;
}

QVariant ChatListModel::data(const QModelIndex& index, int role) const
{
    int pid = index.row();
    auto chat_id = list.at(pid);
    std::shared_ptr<dc::Chat> chat;
    int count;
    QVariant res;
    switch (role) {
    case ListRole::ChatID:
        return chat_id;
    case ListRole::Username:
        chat = dc::Account::getInstance()->getSelectedAccount()->getChat(chat_id);
        return chat->getName();
    case ListRole::Count:
        res = QVariant(dc::Account::getInstance()->getSelectedAccount()->getFreshMsgCnt(chat_id));

        return res;
    default:
        break;
    }

    return "";
}

QString ChatListModel::getName(const QVariant& index) const
{
    // row id
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    int id = index.toInt();
    auto chat_id = list.at(id);
    // qDebug() << "get chat with" + QString::number(chat_id);
    if (chat_id >= 0) {
        auto chat = ctx->getChat(chat_id);
        return chat->getName();
    }
    qDebug() << "Error: can not get chat";
    return "Error";
}

void ChatListModel::onIncomingMessage(uint32_t accountId, int chatId, int msgId)
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    if (ctx->getId() != accountId)
        return;
    auto id = list.indexOf(chatId);
    if (id == -1) {
        qDebug() << "update chat";
        beginInsertRows(QModelIndex(), 0, 0);
        list.prepend(chatId);
        endInsertRows();
    } else if (id >= 0) {
        // move lastest chat to frist
        // beginMoveRows(QModelIndex(), id, id, QModelIndex(), 0);
        // list.removeAt(id);
        // list.prepend(chatId);
        // endMoveRows();
        beginRemoveRows(QModelIndex(), id, id);
        list.removeAt(id);
        endRemoveRows();
        beginInsertRows(QModelIndex(), 0, 0);
        list.prepend(chatId);
        endInsertRows();
    }
    emit updateView();
}

void ChatListModel::onDeleteChat(uint32_t accountId, int chatId)
{
    int find = -1;
    for (int i = 0; i < list.size(); i++) {
        auto cid = list.at(i);
        if (cid == chatId) {
            find = i;
            break;
        }
    }
    // remove row
    beginRemoveRows(QModelIndex(), find, find);
    list.removeAt(find);
    endRemoveRows();
}

void ChatListModel::onConnectyChange(uint32_t accountId, int status)
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    if (ctx->getId() != accountId)
        return;

    int rstat = status >= DC_CONNECTIVITY_WORKING;
    if (m_status != rstat) {
        m_status = rstat;
        emit statusChanged();
    }
}

void ChatListModel::onMsgNotice(uint32_t accountId, int chatId)
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    if (ctx->getId() != accountId)
        return;
    auto pos = list.indexOf(chatId);
    if (pos >= 0) {
        emit dataChanged(this->index(pos), this->index(pos));
    }
}

void ChatListModel::onChatListItemChange(uint32_t accountId, int chatId)
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    if (ctx->getId() != accountId)
        return;
    if (chatId == 0) {
        // render all chat
        beginResetModel();
        list.clear();
        auto plist = ctx->getChatlist(0, "");
        for (int i = 0; i < plist->getChatCount(); i++) {
            list.append(plist->getChatId(i));
        }
        endResetModel();
    } else {
        auto pos = list.indexOf(chatId);
        if (pos == -1) {
            beginInsertRows(QModelIndex(), 0, 0);
            list.prepend(chatId);
            endInsertRows();
        }
    }
}

int ChatListModel::readStatus() const
{
    return m_status;
}
