#include "chatroommodel.h"
#include "chatcore.h"
#include <QDebug>

// static declear
const char32_t u_lock = 0x1F512;
const char32_t u_check = 0x2713;
const char32_t u_false = 0x2717;
const char32_t u_pendding = 0x25EF;

ChatRoomModel::ChatRoomModel(QObject* parent)
    : QAbstractListModel { parent }
{
    mess_count = 10;

    auto acc = dc::Account::getInstance();
    // signal connect
    connect(acc.get(), &dc::Account::incomingMessage, this, &ChatRoomModel::onIncomingMessage);
    connect(acc.get(), &dc::Account::outGoingMessage, this, &ChatRoomModel::onOutGoingMessage);
    connect(this, &ChatRoomModel::updateData, this, &ChatRoomModel::rowChange, Qt::DirectConnection);
    connect(acc.get(), &dc::Account::removeContactFromChat, this, &ChatRoomModel::onContactRemove);
    connect(acc.get(), &dc::Account::messagesDelevery, this, &ChatRoomModel::onMessageDelivered);
    connect(acc.get(), &dc::Account::messageFalse, this, &ChatRoomModel::onMessageDelivered);
    connect(acc.get(), &dc::Account::chatListItemChange, this, &ChatRoomModel::onChatListChange);

    time = new QTimer(this);
    connect(time, &QTimer::timeout, this, &ChatRoomModel::markAsRead); // get chat ip
    auto med = ChatCore::getMediator();
    setChatid(med->getChatId());
    // test = new QAbstractItemModelTester(this, QAbstractItemModelTester::FailureReportingMode::Warning, this);
}

int ChatRoomModel::rowCount(const QModelIndex& parent) const
{
    return mess_count;
}

void ChatRoomModel::setChatid(int newChatid)
{
    if (m_chatid == newChatid)
        return;
    m_chatid = newChatid;
    beginResetModel();
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    chatlist = ctx->getMsgIdList(m_chatid);
    // limit mess display
    current_count = chatlist.count();
    if (current_count < 20) {
        mess_count = chatlist.count();
    } else
        mess_count = 20;

    cur_id = -1;
    // set chat group

    auto chat = ctx->getChat(m_chatid);
    m_group = chat->getType() == DC_CHAT_TYPE_GROUP;
    time->start(3000);
    endResetModel();
}

bool ChatRoomModel::checkSignal(uint32_t accountId, int chatId)
{
    auto id = dc::Account::getInstance()->getSelectedAccount()->getId();
    if (accountId != id || chatId != m_chatid) {
        return false;
    }
    return true;
}

QString ChatRoomModel::displayQrCode(const std::shared_ptr<dc::Lot>& lot, bool& valid) const
{
    int state = lot->getState();
    int id;
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    QString ret;
    valid = true;
    switch (state) {
    case DC_QR_ASK_VERIFYCONTACT:
        id = lot->getId();
        ret = QObject::tr("Chat with : ") + ctx->getContact(id)->getAddr();
        break;
    case DC_QR_ASK_VERIFYGROUP:
        ret = QObject::tr("Chat with group: ") + lot->getText1();
        break;
    default:
        valid = false;
        ret = "";
        break;
    }
    return ret;
}

int ChatRoomModel::loadMoredata()
{

    int pad = 20;
    if (current_count > mess_count + pad)
        mess_count = mess_count + pad;
    else {
        pad = current_count - mess_count;
        mess_count = current_count;
    }

    rstart = 0;
    rend = pad - 1;
    emit this->updateData();
    // for smooth
    if (rend > 1)
        rend = rend - 1;
    return rend;
}

QString ChatRoomModel::chatName()
{
    auto chat = dc::Account::getInstance()->getSelectedAccount()->getChat(m_chatid);
    return chat->getName();
}

void ChatRoomModel::deleteChat()
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    ctx->deleteChat(m_chatid);
    chatlist.clear();
}

bool ChatRoomModel::isGroup()
{

    return m_group;
}

void ChatRoomModel::leaveGroup()
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    ctx->removeContactFromChat(m_chatid, DC_CONTACT_ID_SELF);
    // remove chat
    // ctx->deleteChat(m_chatid);
}

void ChatRoomModel::clearAllMsg()
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    // convert chat list to vector
    QVector<uint32_t> vec(chatlist.size());
    for (auto id : chatlist) {
        vec.append(id);
    }
    beginResetModel();

    // clear all data
    chatlist.clear();
    cur_id = -1;
    mess = nullptr;
    mess_count = 0;
    current_count = 0;

    // delete from deltachat
    ctx->deleteMsgs(vec);
    endResetModel();
}
void ChatRoomModel::onIncomingMessage(uint32_t accountId, int chatId, int msgId)
{

    auto id = dc::Account::getInstance()->getSelectedAccount()->getId();
    if (accountId != id || chatId != m_chatid) {
        return;
    }
    rstart = mess_count;
    rend = mess_count;

    // chatlist = dc::Account::getInstance()->getSelectedAccount()->getMsgIdList(m_chatid);
    chatlist.append(msgId);
    current_count = chatlist.count();
    mess_count++;
    emit this->updateData();
}

void ChatRoomModel::onOutGoingMessage(uint32_t accountId, int chatId, int msgid)
{
    // realy same as incomming
    auto id = dc::Account::getInstance()->getSelectedAccount()->getId();
    if (accountId != id || chatId != m_chatid) {
        return;
    }

    rstart = mess_count;
    rend = mess_count;
    chatlist.append(msgid);
    current_count = chatlist.count();
    mess_count++;
    emit this->updateData();
}

void ChatRoomModel::onContactRemove(uint32_t accountId, int chatId, int contact_id)
{
    auto id = dc::Account::getInstance()->getSelectedAccount()->getId();
    if (accountId != id || chatId != m_chatid) {
        return;
    }
    // get new mesg
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    chatlist = ctx->getMsgIdList(m_chatid);
    int pad = chatlist.size() - current_count;
    rstart = mess_count;
    mess_count += pad;
    rend = mess_count - 1;
    current_count = chatlist.count();
    emit this->updateData();
}

void ChatRoomModel::onMessageDelivered(uint32_t accountId, int chatId, int msg_id)
{
    if (!checkSignal(accountId, chatId)) {
        return;
    }
    auto pos = chatlist.indexOf(msg_id);
    int pad = current_count - pos;
    // force reload message
    cur_id = -1;
    // check if message in display list
    if (pad <= mess_count) {
        auto cpos = index(mess_count - pad);
        emit dataChanged(cpos, cpos, { Role::status });
    }
}

void ChatRoomModel::onChatListChange(uint32_t accountId, int chatId)
{
    if (!checkSignal(accountId, chatId)) {
        return;
    }
    emit this->activeChanged();
}

void ChatRoomModel::rowChange()
{

    // numver off message is display = messcount
    beginInsertRows(QModelIndex(), rstart, rend);
    endInsertRows();
}

void ChatRoomModel::markAsRead()
{
    // get fresh count mess and ark as read all
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    int fcount = ctx->getFreshMsgCnt(m_chatid);
    QVector<uint32_t> listmsg;

    int pad = current_count - 1;
    while (fcount > 0 && pad > -1) {
        auto mess = ctx->getMessage(chatlist.at(pad));
        if (mess->getFromId() != DC_CONTACT_ID_SELF) {
            listmsg.append(chatlist.at(pad));
            fcount--;
        }
        pad--;
    }
    if (listmsg.size() > 0) {
        // qDebug() << "mark as read:" + QString::number(listmsg.size());
        ctx->markseenMsgs(listmsg);
    }
}

static bool check_invite_http(const QString& mess)
{
    return mess.startsWith("https://i.delta.chat");
}
QVariant ChatRoomModel::data(const QModelIndex& index, int role) const
{

    auto lctx = dc::Account::getInstance()->getSelectedAccount();
    // auto list = lctx->getMsgIdList(m_chatid);
    // revert show , lastest mess first
    int id = (current_count - mess_count) + index.row();
    std::shared_ptr<dc::Lot> lot;
    // reduce get message
    if (id != cur_id) {
        mess = lctx->getMessage(chatlist.at(id));
        cur_id = id;
    }

    QString mtext, res;
    int stats;
    bool valid;
    switch (role) {
    case Role::text:
        mtext = mess->getText();
        if (!check_invite_http(mtext))
            return mess->getText();
        else {
            // generate http url
            // sample text: '<html><style type="text/css"></style><a href="http://google.com">google</a></html>'
            lot = lctx->checkQr(mtext);
            res = displayQrCode(lot, valid);
            if (valid)
                res = "<html><style type=\"text/css\"></style><a href=\"" + mtext + "\">" + res + "</a></html>";
            else {
                res = mtext;
            }
            return res;
        }
    case Role::date:
        return mess->getTimestamp();
    case Role::local:
        return mess->getFromId() == DC_CONTACT_ID_SELF;
    case Role::isInfo:
        return mess->isInfo();
    case Role::user:
        id = mess->getFromId();
        if (id != DC_CONTACT_ID_SELF) {
            auto contact = lctx->getContact(id);
            return contact->getAddr();
        }
        break;
    case Role::isProtect:
        // lot = lctx->chatlistGetSummary2(m_chatid, chatlist.at(id));
        return mess->getShowPadlock();
    case Role::status:
        stats = mess->getState();
        if (stats == DC_STATE_OUT_PREPARING || stats == DC_STATE_OUT_DRAFT || stats == DC_STATE_OUT_PENDING) {
            return QString::fromUcs4(&u_pendding, 1);
        } else if (stats == DC_STATE_OUT_FAILED) {
            return QString::fromUcs4(&u_false, 1);
        } else if (stats == DC_STATE_OUT_DELIVERED || stats == DC_STATE_OUT_MDN_RCVD) {
            return QString::fromUcs4(&u_check, 1);
        }
        break;
    default:
        qDebug() << "unknow role";
        break;
    }
    return "";
}

QHash<int, QByteArray> ChatRoomModel::roleNames() const
{
    QHash<int, QByteArray> role;
    role[Role::text] = "$text";
    role[Role::local] = "$local";
    role[Role::date] = "$date";
    role[Role::user] = "$user";
    role[Role::isInfo] = "$isInfo";
    role[Role::isProtect] = "$isProtect";
    role[Role::status] = "$status";
    return role;
}

ChatRoomModel::~ChatRoomModel()
{
    time->stop();
    delete time;
    chatlist.clear();
}

bool ChatRoomModel::getActive() const
{
    auto chat = dc::Account::getInstance()->getSelectedAccount()->getChat(m_chatid);
    return chat->canSend();
}
