#ifndef CHATROOMMODEL_H
#define CHATROOMMODEL_H

#include "chatcore.h"
#include <QAbstractItemModelTester>
#include <QAbstractListModel>
#include <QTimer>

class ChatRoomModel : public QAbstractListModel {
    Q_OBJECT
    Q_PROPERTY(bool active READ getActive NOTIFY activeChanged FINAL)

private:
    int mess_count = 20;
    int current_count;
    int m_chatid;
    // for insert row
    int rstart, rend;
    enum Role {
        text = Qt::UserRole + 1,
        user,
        local,
        date,
        isInfo,
        isProtect,
        status,
    };
    QList<int> chatlist;
    mutable std::shared_ptr<dc::Message> mess;
    mutable int cur_id;

    bool m_group = false;
    // for auto mark as read
    QTimer* time;

public:
    explicit ChatRoomModel(QObject* parent = nullptr);
    ~ChatRoomModel();
    // QAbstractItemModel interface
public:
    int rowCount(const QModelIndex& parent) const;
    QVariant data(const QModelIndex& index, int role) const;

public:
    Q_INVOKABLE int loadMoredata();
    Q_INVOKABLE QString chatName();
    Q_INVOKABLE void deleteChat();
    Q_INVOKABLE bool isGroup();

    Q_INVOKABLE void leaveGroup();
    Q_INVOKABLE void clearAllMsg();
    // QAbstractItemModel interface
    QHash<int, QByteArray> roleNames() const;
    bool getActive() const;

public slots:
    void onIncomingMessage(uint32_t accountId, int chatId, int msgId);
    void onOutGoingMessage(uint32_t accountId, int chatId, int msgid);
    // void onChatListItemChange(uint32_t accountId, int chatId);
    void onContactRemove(uint32_t accountId, int chatId, int contact_id);
    void onMessageDelivered(uint32_t accountId, int chatId, int msg_id);
    void onChatListChange(uint32_t accountId, int chatId);
signals:
    void updateData();

    void activeChanged();

private:
    void rowChange();
    void markAsRead();
    void setChatid(int newChatid);
    bool checkSignal(uint32_t accountId, int chatId);
    QString displayQrCode(const std::shared_ptr<dc::Lot>& lot, bool& valid) const;
};

#endif // CHATROOMMODEL_H
