#include "chatcontact.h"
#include "../chatcore.h"
#include "../chatmediator.h"
#include <QDebug>
ChatContact::ChatContact(QObject* parent)
    : QAbstractListModel { parent }
{
    // m_chatid = ChatCore::getMediator()->getChatId();
    // m_contacts = dc::Account::getInstance()->getSelectedAccount()->getChatContacts(m_chatid);
}

int ChatContact::rowCount(const QModelIndex& parent) const
{
    return m_contacts.size();
}

QVariant ChatContact::data(const QModelIndex& index, int role) const
{
    int row = index.row();
    auto id = m_contacts.at(row);
    auto contact = dc::Account::getInstance()->getSelectedAccount()->getContact(id);
    switch (role) {
    case ChatRole::Contact:
        if (id == DC_CONTACT_ID_SELF)
            return "Me";
        else
            return contact->getAddr();
    case ChatRole::Id:
        return id;
    default:
        return "";
    }
}

QHash<int, QByteArray> ChatContact::roleNames() const
{

    QHash<int, QByteArray> res;
    res[ChatRole::Contact] = "$name";
    res[ChatRole::Id] = "$id";
    return res;
}

int ChatContact::getNuser() const
{
    return m_contacts.size();
}

void ChatContact::removeContact(int id)
{

    auto pos = m_contacts.indexOf(id);
    if (pos > 0) {
        beginRemoveRows(QModelIndex(), pos, pos);
        m_contacts.removeAt(pos);
        endRemoveRows();
        emit this->nuserChanged();

        auto ctx = dc::Account::getInstance()->getSelectedAccount();
        ctx->removeContactFromChat(m_chatId, id);
    }
}

int ChatContact::getChatId() const
{
    return m_chatId;
}

void ChatContact::setChatId(int newChatId)
{
    if (m_chatId == newChatId)
        return;
    m_chatId = newChatId;
    beginResetModel();
    m_contacts = dc::Account::getInstance()->getSelectedAccount()->getChatContacts(m_chatId);
    endResetModel();
    emit chatNameChanged();
    emit chatIdChanged();
}

QString ChatContact::getChatName() const
{
    if (m_chatId >= 0) {
        auto chat = dc::Account::getInstance()->getSelectedAccount()->getChat(m_chatId);
        return chat->getName();
    }
    return "";
}
