#include "contactchecklistmodel.h"
#include "../chatcore.h"
#include <QDebug>

ContactCheckListModel::ContactCheckListModel(QObject* parent)
    : QAbstractListModel { parent }
{
    contact_list = dc::Account::getInstance()->getSelectedAccount()->getContacts(0, "");
    for (int i = 0; i < contact_list.size(); i++) {
        check_list.append(false);
    }
}

int ContactCheckListModel::rowCount(const QModelIndex& parent) const
{
    return contact_list.size();
}

QVariant ContactCheckListModel::data(const QModelIndex& index, int role) const
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    std::shared_ptr<dc::Contact> contact;
    int row = index.row();
    switch (role) {
    case Role::Id:

        return contact_list.at(row);
    case Role::Name:
        contact = ctx->getContact(contact_list.at(row));
        return contact->getAddr();
    case Role::Check:
        return check_list.at(row);
    default:
        return "";
    }
}

bool ContactCheckListModel::setData(const QModelIndex& index, const QVariant& value, int role)
{

    if (role == Role::Check) {

        bool data = value.toBool();
        // qDebug() << "data change to " + QString::number(data);
        int row = index.row();
        check_list.removeAt(row);
        check_list.insert(row, data);
        return true;
    }
    return false;
}

QHash<int, QByteArray> ContactCheckListModel::roleNames() const
{
    QHash<int, QByteArray> rolename;
    rolename[Role::Check] = "$check";
    rolename[Role::Id] = "$id";
    rolename[Role::Name] = "$name";
    return rolename;
}

void ContactCheckListModel::createChat(const QString& name, bool sercure)
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    auto id = ctx->createGroupChat(name, sercure);
    int d = 0;
    if (!sercure) {

        // add memner in check list if not sercure
        for (int i = 0; i < contact_list.size(); i++) {
            if (check_list.at(i)) {
                ctx->addContactToChat(id, contact_list.at(i));
                d++;
            }
        }
    }
}

void ContactCheckListModel::addContactToChat(int id)
{
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    for (int i = 0; i < contact_list.size(); i++) {
        if (check_list.at(i)) {
            ctx->addContactToChat(id, contact_list.at(i));
        }
    }
}

int ContactCheckListModel::editRole()
{
    return Role::Check;
}

int ContactCheckListModel::getChatId() const
{
    return m_id;
}

void ContactCheckListModel::setChatId(int newId)
{
    if (m_id == newId)
        return;
    m_id = newId;
    // remove all contact in chat
    beginResetModel();
    auto ctx = dc::Account::getInstance()->getSelectedAccount();
    contact_list = ctx->getContacts(0, "");
    auto rlist = ctx->getChatContacts(m_id);
    for (auto rid : rlist) {
        int pos = contact_list.indexOf(rid);
        if (pos >= 0)
            contact_list.removeAt(pos);
    }
    for (int i = 0; i < contact_list.size(); i++) {
        check_list.append(false);
    }
    endResetModel();
    emit chatIdChanged();
}
